#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <glob.h>
#include "pvo-block.hh"
#include "pvo-header.hh"
#include "pvo-FITS.hh"
#include "pvo-utils.hh"

using namespace PVO;
using namespace PVO::Header;
using namespace PVO::Block;
using namespace PVO::FITS;
using std::string;


const std::string help_text = 
"Usage:\n\
  1. pvoconesearch ra dec radius\n\
  2. pvoconesearch -h\n\
---------\n\
  1. Prints RA, DEC, DATE-OBS, BEAMSIZE, OFFSET for all FITS files that\n\
     lie within \"radius\" of the position given by ra, dec.\n\
     OFFSET is angular offset from query position.\n\
  2. Prints this help information.\n\n\
 The RA and DEC can be in the form hh:mm:ss.ss etc or in decimal degrees.\n\
 The search radius is in degrees. The beam size is also taken into account,\n\
  so the effective search radius becomes (radius + 0.5*beamFWHM).\n\
 Note that if a keyword is not present, --- is printed.\n\
 FITS files are assumed to have suffixes of .fits or .rf\n";

int main(int argc, char * argv[])
{

  bool verboseflag = false;

  std::vector<std::string> namelist;
  std::string RAstring,DECstring;
  double radius,RAcentre,DECcentre;
  unsigned int i,numColon;
  switch(argc){
  case 4:
    glob_t globbuf1,globbuf2;
    globbuf1.gl_offs = 0;
    globbuf2.gl_offs = 0;
    glob("*.fits", GLOB_DOOFFS, NULL, &globbuf1);
    glob("*.rf", GLOB_DOOFFS, NULL, &globbuf2);
    namelist.resize(globbuf1.gl_pathc+globbuf2.gl_pathc);
    for(i=0;i<globbuf1.gl_pathc+globbuf1.gl_offs;i++) 
      namelist[i] = globbuf1.gl_pathv[i];
    for(i=0;i<globbuf2.gl_pathc+globbuf2.gl_offs;i++) 
      namelist[globbuf1.gl_pathc+i] = globbuf2.gl_pathv[i];

    RAstring = argv[1];
    numColon=0;
    for(i=0;i<RAstring.size();i++) if(RAstring[i]==':') numColon++;
    if(numColon>0) RAcentre = dmsToDec(RAstring)*15.;
    else RAcentre = atof(argv[1]);
    DECstring = argv[2];
    numColon=0;
    for(i=0;i<DECstring.size();i++) if(DECstring[i]==':') numColon++;
    if(numColon>0) DECcentre = dmsToDec(DECstring);
    else DECcentre = atof(argv[2]);

    radius = atof(argv[3]);
    
    std::cout.setf(std::ios::fixed);
    std::cout << std::setprecision(4)
	      << "Searching with: RA = " << RAcentre
	      << " deg, DEC = " << DECcentre
	      << " deg, Radius = " << radius << " deg\n";

    break;
  default:
    if(argc==2 && strcmp(argv[1],"-h")==0)
      std::cout << help_text;
    else
      std::cerr << help_text;
    return 0;
    break;
  }

  int maxsize = 0;
  for(i=0;i<namelist.size();i++) 
    maxsize = std::max(maxsize,int(namelist[i].size()));

  std::cout << std::setfill('-')  
	    << std::setw(maxsize+1+15+15+31+11+13) << "-"
	    << "\n" << std::setfill(' ') 
	    << std::setw(maxsize) << "FILENAME"  << ": "
	    << std::setw(14)      << "RA"        << " "
	    << std::setw(14)      << "DEC"       << " "
	    << std::setw(30)      << "DATE-OBS"  << " "
	    << std::setw(10)      << "BEAM FWHM"  << " "
	    << std::setw(12)      << "OFFSET"    << "\n"
	    << std::setw(maxsize) << ""          << "  "
	    << std::setw(14)      << ""          << " "
	    << std::setw(14)      << ""          << " "
	    << std::setw(30)      << ""          << " "
	    << std::setw(10)      << "[arcmin]"  << " "
	    << std::setw(12)      << "[arcmin]"  << "\n"
	    << std::setfill('-')  
	    << std::setw(maxsize+1+15+15+31+11+13) << "-"
	    << "\n"<< std::setfill(' ') ;

  for(i=0;i<namelist.size();i++){

    PSRFits fitsfile;
    fitsfile.setVerbosity(verboseflag);
    int status = fitsfile.openFile(namelist[i]);
  
    if(status==0){
      fitsfile.gotoPHU();
    
      std::string ra,dec,date;
      std::string missing = "---";
      if(fitsfile.getKeyword<std::string>("RA",ra)!=0) ra = missing;
      if(fitsfile.getKeyword<std::string>("DEC",dec)!=0) dec = missing;
      if(fitsfile.getKeyword<std::string>("DATE-OBS",date)!=0) date = missing;
      float beamradius,bmaj,bmin;
      fitsfile.getKeyword<float>("BMAJ",bmaj);
      fitsfile.getKeyword<float>("BMIN",bmin);
      beamradius = 0.5*(bmaj+bmin)/2.; // half average FWHM, in deg

      if(ra!=missing && dec!=missing){
	double angsep = angSep(dmsToDec(ra)*15, dmsToDec(dec),
			       RAcentre,        DECcentre     );

	if(angsep < radius+beamradius)
	  std::cout << std::setw(maxsize) << namelist[i] << ": "
		    << std::setw(14)      << ra          << " "
		    << std::setw(14)      << dec         << " "
		    << std::setw(30)      << date        << " "
		    << std::setw(10)      << beamradius*2.<< " "
		    << std::setw(12)      << angsep*60.  << "\n";
      
      }
    }
  }

}
